#ifndef MEM_ARENA_H
#define MEM_ARENA_H

#include "aliases.h"
#include "mem_utils.h"
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

typedef struct arena Arena;

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena wrapper macros
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

#define wapp_mem_arena_init(arena_dptr, base_capacity)                         \
  (wapp_mem_arena_init_custom(arena_dptr, base_capacity,                       \
                              WAPP_MEM_ALLOC_RESERVE, false))
#define wapp_mem_arena_init_commit(arena_dptr, base_capacity)                  \
  (wapp_mem_arena_init_custom(arena_dptr, base_capacity,                       \
                              WAPP_MEM_ALLOC_RESERVE | WAPP_MEM_ALLOC_COMMIT,  \
                              false))
#define wapp_mem_arena_init_zero(arena_dptr, base_capacity)                    \
  (wapp_mem_arena_init_custom(arena_dptr, base_capacity,                       \
                              WAPP_MEM_ALLOC_RESERVE, true))
#define wapp_mem_arena_init_commit_and_zero(arena_dptr, base_capacity)         \
  (wapp_mem_arena_init_custom(arena_dptr, base_capacity,                       \
                              WAPP_MEM_ALLOC_RESERVE | WAPP_MEM_ALLOC_COMMIT,  \
                              true))

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena API declarations
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

/**
 * Arena initialisation function. `wapp_mem_arena_init_custom` provides the most
 * control over how the Arena is initialised. Wrapper macros are provided for
 * easier use.
 */
bool wapp_mem_arena_init_custom(Arena **arena, u64 base_capacity,
                                MemAllocFlags flags, bool zero_buffer);
void *wapp_mem_arena_alloc(Arena *arena, u64 size);
void *wapp_mem_arena_alloc_aligned(Arena *arena, u64 size, u64 alignment);
void wapp_mem_arena_clear(Arena *arena);
void wapp_mem_arena_destroy(Arena **arena);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // !MEM_ARENA_H
