#include "mem_arena_allocator.h"
#include "mem_arena.h"

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena Allocator wrappers declarations
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

internal inline void *mem_arena_alloc(u64 size, void *alloc_obj);
internal inline void *mem_arena_alloc_aligned(u64 size, u64 alignment,
                                              void *alloc_obj);

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena Allocator API definitions
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

Allocator wapp_mem_arena_allocator_init_custom(u64 base_capacity,
                                               MemAllocFlags flags,
                                               bool zero_buffer) {
  Allocator allocator = {0};
  bool initialised = wapp_mem_arena_init_custom(
      (Arena **)(&allocator.obj), base_capacity, flags, zero_buffer);
  if (!initialised) {
    return allocator;
  }

  allocator.alloc = mem_arena_alloc;
  allocator.alloc_aligned = mem_arena_alloc_aligned;

  return allocator;
}

void wapp_mem_arena_allocator_clear(Allocator *allocator) {
  wapp_mem_arena_clear((Arena *)(allocator->obj));
}

void wapp_mem_arena_allocator_destroy(Allocator *allocator) {
  wapp_mem_arena_destroy((Arena **)(&(allocator->obj)));
  *allocator = (Allocator){0};
}

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena Allocator wrappers definitions
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

internal inline void *mem_arena_alloc(u64 size, void *alloc_obj) {
  Arena *arena = (Arena *)alloc_obj;
  return wapp_mem_arena_alloc(arena, size);
}

internal inline void *mem_arena_alloc_aligned(u64 size, u64 alignment,
                                              void *alloc_obj) {
  Arena *arena = (Arena *)alloc_obj;
  return wapp_mem_arena_alloc_aligned(arena, size, alignment);
}
