#ifndef MEM_ARENA_ALLOCATOR_H
#define MEM_ARENA_ALLOCATOR_H

#include "aliases.h"
#include "mem_allocator.h"
#include "mem_utils.h"
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena Allocator wrapper macros
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

#define wapp_mem_arena_allocator_init(base_capacity)                           \
  (wapp_mem_arena_allocator_init_custom(base_capacity, WAPP_MEM_ALLOC_RESERVE, \
                                        false))
#define wapp_mem_arena_allocator_init_commit(base_capacity)                    \
  (wapp_mem_arena_allocator_init_custom(                                       \
      base_capacity, WAPP_MEM_ALLOC_RESERVE | WAPP_MEM_ALLOC_COMMIT, false))
#define wapp_mem_arena_allocator_init_zero(base_capacity)                      \
  (wapp_mem_arena_allocator_init_custom(base_capacity, WAPP_MEM_ALLOC_RESERVE, \
                                        true))
#define wapp_mem_arena_allocator_init_commit_and_zero(base_capacity)           \
  (wapp_mem_arena_allocator_init_custom(                                       \
      base_capacity, WAPP_MEM_ALLOC_RESERVE | WAPP_MEM_ALLOC_COMMIT, true))

/***************************************************************************/ //
////////////////////////////////////////////////////////////////////////////////
////// Arena Allocator API declarations
////////////////////////////////////////////////////////////////////////////////
/***************************************************************************/ //

/**
 * Wraps an Arena in an Allocator object. It attempts to initialise the Arena
 * and, if successful, defines the operations supported by it to be used by the
 * Allocator.
 *
 * An Arena allocator only supports normal allocation and aligned allocation.
 * Reallocation, aligned reallocation and freeing aren't implemented.
 *
 * The `wapp_mem_arena_allocator_init_custom` provides the most control over how
 * the Arena is initialised. Wrapper macros are provided for easier use.
 */
Allocator wapp_mem_arena_allocator_init_custom(u64 base_capacity,
                                               MemAllocFlags flags,
                                               bool zero_buffer);
void wapp_mem_arena_allocator_clear(Allocator *allocator);
void wapp_mem_arena_allocator_destroy(Allocator *allocator);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // !MEM_ARENA_ALLOCATOR_H
